#!/usr/bin/lua
-- rpm call
-- debug=true lua -- copy_jdk_configs.lua   --currentjvm "%{uniquesuffix %{nil}}" --jvmdir "%{_jvmdir %{nil}}" --origname "%{name}" --origjavaver "%{javaver}" --arch "%{_arch}"
--test call
-- debug=true lua -- copy_jdk_configs.lua   --currentjvm "java-1.8.0-openjdk-1.8.0.65-3.b17.fc22.x86_64" --jvmdir "/usr/lib/jvm" --origname "java-1.8.0-openjdk" --origjavaver "1.8.0" --arch "x86_64" --jvmDestdir /home/jvanek/Desktop

local M = {}

if (os.getenv("debug") == "true") then
    debug = true;
else
    debug = false;
end

local function debugOneLinePrint(string)
    if (debug) then
        print(string)
    end ;
end

function getPath(str, sep)
    sep = sep or '/'
    return str:match("(.*" .. sep .. ")")
end

function splitToTable(source, pattern)
    local i1 = string.gmatch(source, pattern)
    local l1 = {}
    for i in i1 do
        table.insert(l1, i)
    end
    return l1
end

local function slurp(path)
    local f = io.open(path)
    local s = f:read("*a")
    f:close()
    return s
end

function trim(s)
    return (s:gsub("^%s*(.-)%s*$", "%1"))
end

local function dirWithParents(path)
    local s = ""
    local dirs = splitToTable(path, "[^/]+")
    for i, d in pairs(dirs) do
        if (i == #dirs) then
            break
        end
        s = s .. "/" .. d
        local stat2 = posix.stat(s, "type");
        if (stat2 == nil) then
            debugOneLinePrint(s .. " does not exists, creating")
            if (not dry) then
                posix.mkdir(s)
            end
        else
            debugOneLinePrint(s .. " exists,not creating")
        end
    end
end


-- main function,
-- formelry main body
-- move to function resolved
-- https://bugzilla.redhat.com/show_bug.cgi?id=1892224
-- for readability not indented, todo, indent once tuned

function M.mainProgram(arg)
    debugOneLinePrint("cjc: lua debug on")
    local caredFiles = { "jre/lib/calendars.properties",
                         "jre/lib/content-types.properties",
                         "jre/lib/flavormap.properties",
                         "jre/lib/logging.properties",
                         "jre/lib/net.properties",
                         "jre/lib/psfontj2d.properties",
                         "jre/lib/sound.properties",
                         "jre/lib/deployment.properties",
                         "jre/lib/deployment.config",
                         "jre/lib/security/US_export_policy.jar",
                         "jre/lib/security/unlimited/US_export_policy.jar",
                         "jre/lib/security/limited/US_export_policy.jar",
                         "jre/lib/security/policy/unlimited/US_export_policy.jar",
                         "jre/lib/security/policy/limited/US_export_policy.jar",
                         "jre/lib/security/java.policy",
                         "jre/lib/security/java.security",
                         "jre/lib/security/local_policy.jar",
                         "jre/lib/security/unlimited/local_policy.jar",
                         "jre/lib/security/limited/local_policy.jar",
                         "jre/lib/security/policy/unlimited/local_policy.jar",
                         "jre/lib/security/policy/limited/local_policy.jar",
                         "jre/lib/security/nss.cfg",
                         "jre/lib/security/cacerts",
                         "jre/lib/security/blacklisted.certs",
                         "jre/lib/security/jssecacerts",
                         "jre/lib/security/trusted.certs",
                         "jre/lib/security/trusted.jssecerts",
                         "jre/lib/security/trusted.clientcerts",
                         "jre/lib/ext",
                         "jre/lib/security/blacklist",
                         "jre/lib/security/javaws.policy",
                         "jre/lib/security/nss.fips.cfg",
                         "lib/security",
                         "conf",
                         "lib/ext" }

    -- before import to allow run from spec
    if (arg[1] == "--list") then
        for i, file in pairs(caredFiles) do
            print(file)
        end
        return 0;
    end

    -- yum install lua-posix
    local posix = require "posix"

    -- the one we are installing
    local currentjvm = nil
    local jvmdir = nil
    local jvmDestdir = nil
    local origname = nil
    local origjavaver = nil
    local arch = nil
    local temp = nil;
    local dry = false;

    for i = 1, #arg, 2 do
        if (arg[i] == "--help" or arg[i] == "-h") then
            print("all but jvmDestdir and debug are mandatory")
            print("  --currentjvm")
            print("    NVRA of currently installed java")
            print("  --jvmdir")
            print("    Directory where to find this kind of virtual machine. Generally /usr/lib/jvm")
            print("  --origname")
            print("    convinient switch to determine jdk. Generally java-1.X.0-vendor")
            print("  --origjavaver")
            print("    convinient switch to determine jdk's version. Generally 1.X.0")
            print("  --arch")
            print("    convinient switch to determine jdk's arch")
            print("  --jvmDestdir")
            print("    Migration/testing switch. Target Mostly same as jvmdir, but you may wont to copy ouside it.")
            print("  --debug or $debug")
            print("    Enables printing out whats going on. true/false. False by default")
            print("  --temp")
            print("    optional file to save intermediate result - directory configs were copied from")
            print("  --dry")
            print("    true/fase if true, then no changes will be written to disk except one tmp file. False by default")
            print("  **** specil parasm ****")
            print("  --list")
            print("    if present on cmdline, list all cared files and exists")
            os.exit(0)
        end
        if (arg[i] == "--currentjvm") then
            currentjvm = arg[i + 1]
        end
        if (arg[i] == "--jvmdir") then
            jvmdir = arg[i + 1]
        end
        if (arg[i] == "--origname") then
            origname = arg[i + 1]
        end
        if (arg[i] == "--origjavaver") then
            origjavaver = arg[i + 1]
        end
        if (arg[i] == "--arch") then
            arch = arg[i + 1]
        end
        if (arg[i] == "--jvmDestdir") then
            jvmDestdir = arg[i + 1]
        end
        if (arg[i] == "--debug") then
            --no string, boolean, workaround
            if (arg[i + 1] == "true") then
                debug = true
            end
        end
        if (arg[i] == "--dry") then
            --no string, boolean, workaround
            if (arg[i + 1] == "true") then
                dry = true
            end
        end
        if (arg[i] == "--temp") then
            temp = arg[i + 1]
        end
    end

    if (jvmDestdir == nil) then
        jvmDestdir = jvmdir
    end

    if (debug) then
        print("--currentjvm:");
        print(currentjvm);
        print("--jvmdir:");
        print(jvmdir);
        print("--jvmDestdir:");
        print(jvmDestdir);
        print("--origname:");
        print(origname);
        print("--origjavaver:");
        print(origjavaver);
        print("--arch:");
        print(arch);
        print("--debug:");
        print(debug);
    end

    --trasnform substitute names to lua patterns
    local name = string.gsub(string.gsub(origname, "%-", "%%-"), "%.", "%%.")
    local javaver = string.gsub(origjavaver, "%.", "%%.")

    local jvms = { }

    debugOneLinePrint("started")

    foundJvms = posix.dir(jvmdir);
    if (foundJvms == nil) then
        debugOneLinePrint("no, or nothing in " .. jvmdir .. " exit")
        return
    end

    debugOneLinePrint("found " .. #foundJvms .. " jvms")

    for i, p in pairs(foundJvms) do
        -- regex similar to %{_jvmdir}/%{name}-%{javaver}*%{_arch} bash command
        if (string.find(p, name .. "%-" .. javaver .. ".*" .. arch) ~= nil) then
            debugOneLinePrint("matched:  " .. p)
            if (currentjvm == p) then
                debugOneLinePrint("this jdk is already installed. exiting lua script")
                return
            end ;
            if (string.match(p, ".*-debug$")) then
                print(p .. " matched but seems to be debug variant. Skipping")
            else
                table.insert(jvms, p)
            end
        else
            debugOneLinePrint("NOT matched:  " .. p)
        end
    end

    if (#jvms <= 0) then
        debugOneLinePrint("no matching jdk in " .. jvmdir .. " exit")
        return
    end ;

    debugOneLinePrint("matched " .. #jvms .. " jdk in " .. jvmdir)

    --full names are like java-1.7.0-openjdk-1.7.0.60-2.4.5.1.fc20.x86_64
    table.sort(jvms, function(a, b)
        -- version-sort
        -- split on non word: . -
        local l1 = splitToTable(a, "[^%.-]+")
        local l2 = splitToTable(b, "[^%.-]+")
        for x = 1, math.min(#l1, #l2) do
            local l1x = tonumber(l1[x])
            local l2x = tonumber(l2[x])
            if (l1x ~= nil and l2x ~= nil) then
                --if hunks are numbers, go with them
                if (l1x < l2x) then
                    return true;
                end
                if (l1x > l2x) then
                    return false;
                end
            else
                if (l1[x] < l2[x]) then
                    return true;
                end
                if (l1[x] > l2[x]) then
                    return false;
                end
            end
            -- if hunks are equals then move to another pair of hunks
        end
        return a < b

    end)

    if (debug) then
        print("sorted lsit of jvms")
        for i, file in pairs(jvms) do
            print(file)
        end
    end

    latestjvm = jvms[#jvms]

    if (temp ~= nil) then
        src = jvmdir .. "/" .. latestjvm
        debugOneLinePrint("temp declared as " .. temp .. " saving used dir of " .. src)
        file = io.open(temp, "w")
        file:write(src)
        file:close()
    end

    local readlinkOutput = os.tmpname()

    for i, file in pairs(caredFiles) do
        local SOURCE = jvmdir .. "/" .. latestjvm .. "/" .. file
        local DEST = jvmDestdir .. "/" .. currentjvm .. "/" .. file
        debugOneLinePrint("going to copy " .. SOURCE)
        debugOneLinePrint("to  " .. DEST)
        local stat1 = posix.stat(SOURCE, "type");
        if (stat1 ~= nil) then
            debugOneLinePrint(SOURCE .. " exists")
            dirWithParents(DEST)
            -- Copy with -a to keep everything intact
            local exe = "cp" .. " -ar " .. SOURCE .. " " .. DEST
            local linkExe = "readlink" .. " -f " .. SOURCE .. " > " .. readlinkOutput
            debugOneLinePrint("executing " .. linkExe)
            os.remove(readlinkOutput)
            os.execute(linkExe)
            local link = trim(slurp(readlinkOutput))
            debugOneLinePrint("  ...link is " .. link)
            if (not ((link) == (SOURCE))) then
                debugOneLinePrint("WARNING link " .. link .. " where file " .. SOURCE .. " expected!")
                debugOneLinePrint("Will try to copy link target rather then link itself!")
                --replacing  any NVRA by future NVRA (still execting to have NVRA for any multiple-installable targets
                -- lua stubbornly consider dash as inteval. Replacing by dot to match X-Y more correct as X.Y rather then not at all
                local linkDest = string.gsub(link, latestjvm:gsub("-", "."), currentjvm)
                debugOneLinePrint("attempting to copy " .. link .. " to " .. linkDest)
                if (link == linkDest) then
                    debugOneLinePrint("Those are identical files! Nothing to do!")
                else
                    local exe2 = "cp" .. " -ar " .. link .. " " .. linkDest
                    dirWithParents(linkDest)
                    debugOneLinePrint("executing " .. exe2)
                    if (not dry) then
                        os.execute(exe2)
                    end
                end
            else
                debugOneLinePrint("executing " .. exe)
                if (not dry) then
                    os.execute(exe)
                end
            end
        else
            debugOneLinePrint(SOURCE .. " does not exists")
        end
    end

end --unindented main function

if (arg == nil) then
    debugOneLinePrint("arg variable is nil, you have to call mainProgram manually") -- this can actually not be invoked, as the debug is set via arg
else
    M.mainProgram(arg)
end

return M
;jre/lib/security/nss.fips.cfg&#34;,
                         &#34;lib/security&#34;,
                         &#34;conf&#34;,
                         &#34;lib/ext&#34; }

    -- before import to allow run from spec
    if (arg[1] == &#34;--list&#34;) then
        for i, file in pairs(caredFiles) do
            print(file)
        end
        return 0;
    end

    -- yum install lua-posix
    local posix = require &#34;posix&#34;

    -- the one we are installing
    local currentjvm = nil
    local jvmdir = nil
    local jvmDestdir = nil
    local origname = nil
    local origjavaver = nil
    local arch = nil
    local temp = nil;
    local dry = false;

    for i = 1, #arg, 2 do
        if (arg[i] == &#34;--help&#34; or arg[i] == &#34;-h&#34;) then
            print(&#34;all but jvmDestdir and debug are mandatory&#34;)
            print(&#34;  --currentjvm&#34;)
            print(&#34;    NVRA of currently installed java&#34;)
            print(&#34;  --jvmdir&#34;)
            print(&#34;    Directory where to find this kind of virtual machine. Generally /usr/lib/jvm&#34;)
            print(&#34;  --origname&#34;)
            print(&#34;    convinient switch to determine jdk. Generally java-1.X.0-vendor&#34;)
            print(&#34;  --origjavaver&#34;)
            print(&#34;    convinient switch to determine jdk&#39;s version. Generally 1.X.0&#34;)
            print(&#34;  --arch&#34;)
            print(&#34;    convinient switch to determine jdk&#39;s arch&#34;)
            print(&#34;  --jvmDestdir&#34;)
            print(&#34;    Migration/testing switch. Target Mostly same as jvmdir, but you may wont to copy ouside it.&#34;)
            print(&#34;  --debug or $debug&#34;)
            print(&#34;    Enables printing out whats going on. true/false. False by default&#34;)
            print(&#34;  --temp&#34;)
            print(&#34;    optional file to save intermediate result - directory configs were copied from&#34;)
            print(&#34;  --dry&#34;)
            print(&#34;    true/fase if true, then no changes will be written to disk except one tmp file. False by default&#34;)
            print(&#34;  **** specil parasm ****&#34;)
            print(&#34;  --list&#34;)
            print(&#34;    if present on cmdline, list all cared files and exists&#34;)
            os.exit(0)
        end
        if (arg[i] == &#34;--currentjvm&#34;) then
            currentjvm = arg[i + 1]
        end
        if (arg[i] == &#34;--jvmdir&#34;) then
            jvmdir = arg[i + 1]
        end
        if (arg[i] == &#34;--origname&#34;) then
            origname = arg[i + 1]
        end
        if (arg[i] == &#34;--origjavaver&#34;) then
            origjavaver = arg[i + 1]
        end
        if (arg[i] == &#34;--arch&#34;) then
            arch = arg[i + 1]
        end
        if (arg[i] == &#34;--jvmDestdir&#34;) then
            jvmDestdir = arg[i + 1]
        end
        if (arg[i] == &#34;--debug&#34;) then
            --no string, boolean, workaround
            if (arg[i + 1] == &#34;true&#34;) then
                debug = true
            end
        end
        if (arg[i] == &#34;--dry&#34;) then
            --no string, boolean, workaround
            if (arg[i + 1] == &#34;true&#34;) then
                dry = true
            end
        end
        if (arg[i] == &#34;--temp&#34;) then
            temp = arg[i + 1]
        end
    end

    if (jvmDestdir == nil) then
        jvmDestdir = jvmdir
    end

    if (debug) then
        print(&#34;--currentjvm:&#34;);
        print(currentjvm);
        print(&#34;--jvmdir:&#34;);
        print(jvmdir);
        print(&#34;--jvmDestdir:&#34;);
        print(jvmDestdir);
        print(&#34;--origname:&#34;);
        print(origname);
        print(&#34;--origjavaver:&#34;);
        print(origjavaver);
        print(&#34;--arch:&#34;);
        print(arch);
        print(&#34;--debug:&#34;);
        print(debug);
    end

    --trasnform substitute names to lua patterns
    local name = string.gsub(string.gsub(origname, &#34;%-&#34;, &#34;%%-&#34;), &#34;%.&#34;, &#34;%%.&#34;)
    local javaver = string.gsub(origjavaver, &#34;%.&#34;, &#34;%%.&#34;)

    local jvms = { }

    debugOneLinePrint(&#34;started&#34;)

    foundJvms = posix.dir(jvmdir);
    if (foundJvms == nil) then
        debugOneLinePrint(&#34;no, or nothing in &#34; .. jvmdir .. &#34; exit&#34;)
        return
    end

    debugOneLinePrint(&#34;found &#34; .. #foundJvms .. &#34; jvms&#34;)

    for i, p in pairs(foundJvms) do
        -- regex similar to %{_jvmdir}/%{name}-%{javaver}*%{_arch} bash command
        if (string.find(p, name .. &#34;%-&#34; .. javaver .. &#34;.*&#34; .. arch) ~= nil) then
            debugOneLinePrint(&#34;matched:  &#34; .. p)
            if (currentjvm == p) then
                debugOneLinePrint(&#34;this jdk is already installed. exiting lua script&#34;)
                return
            end ;
            if (string.match(p, &#34;.*-debug$&#34;)) then
                print(p .. &#34; matched but seems to be debug variant. Skipping&#34;)
            else
                table.insert(jvms, p)
            end
        else
            debugOneLinePrint(&#34;NOT matched:  &#34; .. p)
        end
    end

    if (#jvms &lt;= 0) then
        debugOneLinePrint(&#34;no matching jdk in &#34; .. jvmdir .. &#34; exit&#34;)
        return
    end ;

    debugOneLinePrint(&#34;matched &#34; .. #jvms .. &#34; jdk in &#34; .. jvmdir)

    --full names are like java-1.7.0-openjdk-1.7.0.60-2.4.5.1.fc20.x86_64
    table.sort(jvms, function(a, b)
        -- version-sort
        -- split on non word: . -
        local l1 = splitToTable(a, &#34;[^%.-]+&#34;)
        local l2 = splitToTable(b, &#34;[^%.-]+&#34;)
        for x = 1, math.min(#l1, #l2) do
            local l1x = tonumber(l1[x])
            local l2x = tonumber(l2[x])
            if (l1x ~= nil and l2x ~= nil) then
                --if hunks are numbers, go with them
                if (l1x &lt; l2x) then
                    return true;
                end
                if (l1x &gt; l2x) then
                    return false;
                end
            else
                if (l1[x] &lt; l2[x]) then
                    return true;
                end
                if (l1[x] &gt; l2[x]) then
                    return false;
                end
            end
            -- if hunks are equals then move to another pair of hunks
        end
        return a &lt; b

    end)

    if (debug) then
        print(&#34;sorted lsit of jvms&#34;)
        for i, file in pairs(jvms) do
            print(file)
        end
    end

    latestjvm = jvms[#jvms]

    if (temp ~= nil) then
        src = jvmdir .. &#34;/&#34; .. latestjvm
        debugOneLinePrint(&#34;temp declared as &#34; .. temp .. &#34; saving used dir of &#34; .. src)
        file = io.open(temp, &#34;w&#34;)
        file:write(src)
        file:close()
    end

    local readlinkOutput = os.tmpname()

    for i, file in pairs(caredFiles) do
        local SOURCE = jvmdir .. &#34;/&#34; .. latestjvm .. &#34;/&#34; .. file
        local DEST = jvmDestdir .. &#34;/&#34; .. currentjvm .. &#34;/&#34; .. file
        debugOneLinePrint(&#34;going to copy &#34; .. SOURCE)
        debugOneLinePrint(&#34;to  &#34; .. DEST)
        local stat1 = posix.stat(SOURCE, &#34;type&#34;);
        if (stat1 ~= nil) then
            debugOneLinePrint(SOURCE .. &#34; exists&#34;)
            dirWithParents(DEST)
            -- Copy with -a to keep everything intact
            local exe = &#34;cp&#34; .. &#34; -ar &#34; .. SOURCE .. &#34; &#34; .. DEST
            local linkExe = &#34;readlink&#34; .. &#34; -f &#34; .. SOURCE .. &#34; &gt; &#34; .. readlinkOutput
            debugOneLinePrint(&#34;executing &#34; .. linkExe)
            os.remove(readlinkOutput)
            os.execute(linkExe)
            local link = trim(slurp(readlinkOutput))
            debugOneLinePrint(&#34;  ...link is &#34; .. link)
            if (not ((link) == (SOURCE))) then
                debugOneLinePrint(&#34;WARNING link &#34; .. link .. &#34; where file &#34; .. SOURCE .. &#34; expected!&#34;)
                debugOneLinePrint(&#34;Will try to copy link target rather then link itself!&#34;)
                --replacing  any NVRA by future NVRA (still execting to have NVRA for any multiple-installable targets
                -- lua stubbornly consider dash as inteval. Replacing by dot to match X-Y more correct as X.Y rather then not at all
                local linkDest = string.gsub(link, latestjvm:gsub(&#34;-&#34;, &#34;.&#34;), currentjvm)
                debugOneLinePrint(&#34;attempting to copy &#34; .. link .. &#34; to &#34; .. linkDest)
                if (link == linkDest) then
                    debugOneLinePrint(&#34;Those are identical files! Nothing to do!&#34;)
                else
                    local exe2 = &#34;cp&#34; .. &#34; -ar &#34; .. link .. &#34; &#34; .. linkDest
                    dirWithParents(linkDest)
                    debugOneLinePrint(&#34;executing &#34; .. exe2)
                    if (not dry) then
                        os.execute(exe2)
                    end
                end
            else
                debugOneLinePrint(&#34;executing &#34; .. exe)
                if (not dry) then
                    os.execute(exe)
                end
            end
        else
            debugOneLinePrint(SOURCE .. &#34; does not exists&#34;)
        end
    end

end --unindented main function

if (arg == nil) then
    debugOneLinePrint(&#34;arg variable is nil, you have to call mainProgram manually&#34;) -- this can actually not be invoked, as the debug is set via arg
else
    M.mainProgram(arg)
end

return M
</code></pre>
  </div>
 </div> <!-- end .card-->

</div>
</div>
</div>
    </div>

        <div class="footer pt-4 text-white">
        <div class="container">
            <div class="d-flex align-items-center">
                <div>
                    <div>Powered by <a href="https://pagure.io/pagure" class="notblue">Pagure</a> 5.13.3</div>
                    <div>
                        <a href="https://docs.pagure.org/pagure/usage/index.html" class="notblue">Documentation</a> &bull;
                        <a href="https://pagure.io/pagure/new_issue" class="notblue">File an Issue</a> &bull;
                        <a href="/about">About</a> &bull;
                        <a href="/ssh_info" class="notblue">SSH Hostkey/Fingerprint</a>
                    </div>
                </div>
                <div class="ml-auto text-right">
                    <div>&copy; Red Hat, Inc. and others.</div>
                </div>
            </div>
        </div>
    </div>


    <script type="text/javascript" nonce="jX4DCHswUEpIUM5ppoYzEgvOE" src="/static/vendor/jquery/jquery.min.js?version=5.13.3"></script>

    <script src="/static/vendor/bootstrap/bootstrap.bundle.min.js?version=5.13.3"></script>

    <script type="text/javascript" nonce="jX4DCHswUEpIUM5ppoYzEgvOE">
      $('[data-toggle="tooltip"]').tooltip({placement : 'bottom'});
      $(".cancel_btn").click(function() {
        history.back();
      });
    </script>

<script type="text/javascript"  nonce="jX4DCHswUEpIUM5ppoYzEgvOE" src="/static/vendor/lazyload/lazyload.min.js?version=5.13.3"></script>

<script type="text/javascript" nonce="jX4DCHswUEpIUM5ppoYzEgvOE">
window.addEventListener("load", function(event) {
    lazyload();
});
</script>

<script type="text/javascript" nonce="jX4DCHswUEpIUM5ppoYzEgvOE">
$("#giturl-toggle").on('click', function(event){
  event.stopPropagation();
  $("#giturl-more").toggle();
  $("#giturl-toggle").hide();
})

$(".fork_project_btn").click(function() {
  $('#fork_project').submit();
});

$(".select-on-focus").on("focus", function() {
  $(this).select();
});

</script>

<script type="text/javascript" nonce="jX4DCHswUEpIUM5ppoYzEgvOE" src="/static/vendor/highlight.js/highlight.pack.js?version=5.13.3"></script>
<script type="text/javascript" nonce="jX4DCHswUEpIUM5ppoYzEgvOE" src="/static/vendor/highlightjs-line-numbers/highlightjs-line-numbers.min.js?version=5.13.3"></script>
<script type="text/javascript" nonce="jX4DCHswUEpIUM5ppoYzEgvOE" src="/static/vendor/highlight.js/spec.js?version=5.13.3"></script>

<script type="text/javascript" nonce="jX4DCHswUEpIUM5ppoYzEgvOE">
  $(document).ready(function() {
  $('.fork_project_btn').click($("[name=fork_project]").submit);

  $('pre.syntaxhighlightblock code').each(function(i, block) {
    hljs.highlightBlock(block);
    hljs.lineNumbersBlock(block);
  });

  var cls = "highlighted-line";
  var lines = location.hash.substr(2).split('-').map(function (x) { return parseInt(x, 10) });
  if (! isNaN(lines[0]))
  {
    for (var i = lines[lines.length - 1]; i >= lines[0]; i--) {
      $('#_' + i).parent().parent().addClass(cls);
    }
    setTimeout(function(){
      $("#_" + lines[0]).get(0).scrollIntoView({behavior: "instant", block: "start", inline: "nearest"});
    }, 50);
  }
});
</script>

<script type="text/javascript" nonce="jX4DCHswUEpIUM5ppoYzEgvOE">

  function updateHighlight() {
    var cls = "highlighted-line";
    $('.' + cls).removeClass(cls)
    if (location.hash !== '') {
      var lines = location.hash.substr(2).split('-').map(function (x) { return parseInt(x, 10) });
      for (var i = lines[lines.length - 1]; i >= lines[0]; i--) {
        $('[data-line-number=' + i + ']').closest('tr').addClass(cls);
      }
      return lines;
    }
    return [];
  }
  $(window).on('hashchange', updateHighlight);
  var selected = [];
  $("[data-line-number]").click(function (ev) {
    var line = $(this).attr('data-line-number');
    if (ev.shiftKey) {
      selected = selected.slice(-1).concat(line);
    } else {
      selected = [line];
    }

    var hash = '_' + selected[0];
    if (selected.length === 2) {
      hash = '_' + Math.min(selected[0], selected[1]) + '-' + Math.max(selected[0], selected[1]);
    }
    window.location.hash = hash;
    return false;
  });

</script>


</body>
</html>