#!/bin/bash
config=$1
target=$2

debug(){
  if [ "x$debug" == "xtrue" ] ; then
    echo "$@"
  fi
}

debug "cjc: bash debug is on"

cmdvDebug() {
  if [ "x$debug" == "xtrue" ] ; then
    "$@" -v
  else
    "$@" 1>/dev/null 2>&1
  fi
}

mvDebug() {
  cmdvDebug mv "$@"
}

rmDebug() {
  cmdvDebug rm "$@"
}

rmdirDebug() {
 cmdvDebug rmdir "$@"
}

#we should be pretty strict about removing once used (even "used" [with fail]) config file, as it may corrupt another installation
clean(){
  debug "cleanup: removing $config"
  rmDebug -rf $config
}

if [ "x" == "x$config" ] ; then
  debug "no config file specified"
  exit 1
fi

if [ ! -f  "$config" ] ; then
  debug "$config file do not exists"
  # expected case, when no migration happened
  exit 0
fi 

if [ "x" == "x$target" ] ; then
  debug "no target dir specified"
  clean
  exit 2
fi

if [ ! -d  "$target" ] ; then
  debug "$target is not directory"
  clean
  exit 22
fi 

source=`cat $config` 

if [ "x" == "x$source" ] ; then
  debug "no information in $config"
  clean
  exit 3
fi

if [ ! -d  "$source" ] ; then
  debug "$source from $config is not directory"
  clean
  exit 33
fi 


listLinks(){
  find $1 -type l -print0 | xargs -0 ls -ld | sed "s; \+-> \+;_->_;g" | sed "s;.* $1;$1;"
}

printIfExists(){
  if [ -e $ffileCandidate ] ; then
    echo $1
  else
    # stdout can be captured, therefore stderr
    debug "skipping not-existing link-target-dir $1" 1>&2
  fi
}

createListOfLinksTargetsDirectories(){
  pushd $source >/dev/null 2>&1 
    local links=`listLinks $1`
    for x in $links ; do 
      echo "$x" | grep "jre-abrt" > /dev/null
      if [ $? -eq 0 ] ; then
        continue
      fi
      local ffileCandidate=$(echo $x | sed "s/.*_->_//") ;
# ignoring relative paths as they may lead who know where later   
# there can be simlink relative to position, so push is not catching all
      if [ "$ffileCandidate" != "${ffileCandidate#/}" ] ; then
        if [ -d $ffileCandidate ] ; then
# should we accept the links to directories themselves?
          printIfExists $ffileCandidate
        else
          printIfExists `dirname $ffileCandidate`
        fi
      fi
    done | sort | uniq
  popd >/dev/null 2>&1 
}

sourceLinks=`listLinks $source`
targetLinks=`listLinks $target`
sourceLinksDirsTarget=`createListOfLinksTargetsDirectories  $source`
targetLinksDirsTarget=`createListOfLinksTargetsDirectories  $target`

debug "source: $source"
debug "target: $target"

debug "sourceLinks:
$sourceLinks"
debug "targetLinks:
$targetLinks"

debug "sourceLinksDirsTarget:
$sourceLinksDirsTarget"
debug "targetLinksDirsTarget:
$targetLinksDirsTarget"

sourceSearchPath="$source $sourceLinksDirsTarget"
targetSearchPath="$target $targetLinksDirsTarget"

work(){
  if [ "X$1" == "Xrpmnew" -o "X$1" == "Xrpmorig" ] ; then
    debug "Working with $1 (1)"
  else
    debug "unknown parameter: $1"
    return 1
  fi

  local files=`find $targetSearchPath | grep "\\.$1$"`
  for file in $files ; do
    local sf1=`echo $file | sed "s/\\.$1$//"`
    local sf2=`echo $sf1 | sed "s/$targetName/$srcName/"`
    # was file modified in origianl installation?
    rpm -Vf $source | grep -q $sf2
    if [ $? -gt 0 ] ; then
     if [ "X$1" == "Xrpmnew" ] ; then
       debug "$sf2 was NOT modified, removing possibly corrupted $sf1 and renaming from $file"
       mvDebug -f $file $sf1
       if [ $? -eq 0 ] ; then
         echo "restored $file to $sf1"
       else
         debug "FAILED to restore $file to $sf1"
       fi
    fi
     if [ "X$1" == "Xrpmorig" ] ; then
       debug "$sf2 was NOT modified, removing possibly corrupted $file"
       rmDebug $file
    fi
    else
     debug "$sf2 was modified, keeping $file, and removing the duplicated original"
     # information is now backuped, in new directory anyway. Removing future rpmsave to allow rpm -e
     rmDebug -f $sf2
     # or its corresponding backup
     rmDebug -f $sf2.$1
    fi
done
}


srcName=`basename $source`
targetName=`basename $target`

work rpmnew
work rpmorig

debug "Working with rpmorig (2)"
# simply moving old rpmsaves to new dir
# fix for config (replace) leftovers
files=`find $sourceSearchPath | grep "\\.rpmorig$"`
  for file in $files ; do
    rpmsaveTarget=`echo $file | sed "s/$srcName/$targetName/"`
    debug "relocating $file to $rpmsaveTarget"
    if [ -e $rpmsaveTarget ] ; then
      rmDebug $file
    else
      mvDebug $file $rpmsaveTarget
    fi
  done

debug "Working with rpmsave (1)"
files=`find $sourceSearchPath | grep "\\.rpmsave$"`
  for file in $files ; do
    rpmsaveTarget=`echo $file | sed "s/$srcName/$targetName/"`
    debug "relocating $file to $rpmsaveTarget"
    if [ -e $rpmsaveTarget ] ; then
      rmDebug $file
    else
      mvDebug $file $rpmsaveTarget
    fi
  done


#warning: file /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.131-11.b12.el7.x86_64-debug/jre/lib/applet: remove failed: No such file or directory
#warning: file /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.131-11.b12.el7.x86_64-debug/jre/lib/amd64/client: remove failed: No such file or directory
#warning: file /usr/lib/jvm/java-1.7.0-openjdk-1.7.0.171-2.6.13.2.el7.x86_64/jre/lib/amd64/xawt: remove failed: No such file or directory
#those dirs might be mepty by installtion, filling to not be rmeoved later
#use exported CJC_BLACKDIRS_ADD to extend it in runtime/spec file
blackdirs=""
internal_blackdirs="jre/lib/applet jre/lib/*/client jre/lib/locale/*/LC_MESSAGES jre/lib/*/xawt jre/javaws properties/version properties jre/lib/endorsed jre/lib/boot lib/missioncontrol/p2/org.eclipse.equinox.p2.engine/profileRegistry/JMC.profile/.data"
for x in $internal_blackdirs $CJC_BLACKDIRS_ADD ; do 
  blackdirs="$blackdirs $source/$x"
done

for blackdir in $blackdirs; do
  if [ -e $blackdir ] ; then
    debug "nasty $blackdir  exists, filling"
    touch $blackdir/C-J-C_placeholder
  else
    debug "nasty $blackdir  DONT exists, ignoring"
  fi
done

debug "cleaning legacy leftowers"
if [ "x$debug" == "xtrue" ] ; then
  find $sourceSearchPath -empty -type d -delete
else
  find $sourceSearchPath -empty -type d -delete 2>/dev/null >/dev/null
fi
rmdirDebug $sourceSearchPath

# and remove placeholders
for blackdir in $blackdirs; do
  if [ -e $blackdir ] ; then
    debug "nasty $blackdir  exists, cleaning placeholder"
    rmDebug $blackdir/C-J-C_placeholder
  else
    debug "nasty $blackdir  DONT exists, ignoring again"
  fi
done

clean

exit 0
se">
        Commit: <span class="font-weight-bold">88d3ed89f30d8b0eb4877d860fa8d951f224f156</span>
        </a>
    </div>
  </div>
  </div>
    <div class="card mb-3">
      <div class="card-header">
        <ol class="breadcrumb p-0 bg-transparent mb-0">
          <li class="breadcrumb-item">
            <a href="/copy_jdk_configs/tree/88d3ed89f30d8b0eb4877d860fa8d951f224f156">
              <span class="fa fa-random">
              </span>&nbsp; 88d3ed89f30d8b0eb4877d860fa8d951f224f156
            </a>
          </li>
          <li class="active breadcrumb-item">
            <span class="fa fa-file" data-glyph="">
            </span>&nbsp; copy_jdk_configs_fixFiles.sh
          </li>
        </ol>
      </div>

  <div class="card-body p-0">
            <div class="bg-light border text-right pr-2">
                <form class="btn btn-sm" method="POST" name="fork_project"
                    action="/fork_edit/copy_jdk_configs/edit/88d3ed89f30d8b0eb4877d860fa8d951f224f156/f/copy_jdk_configs_fixFiles.sh">
                    <button class="btn btn-sm btn-secondary fork_project_btn">
                            Fork and Edit
                    </button>
                    
                </form>

                <a class="btn btn-secondary btn-sm" href="/copy_jdk_configs/blob/88d3ed89f30d8b0eb4877d860fa8d951f224f156/f/copy_jdk_configs_fixFiles.sh" title="View as blob">Blob</a>

                <a class="btn btn-secondary btn-sm" href="/copy_jdk_configs/blame/copy_jdk_configs_fixFiles.sh?identifier=88d3ed89f30d8b0eb4877d860fa8d951f224f156" title="View git blame">Blame</a>

                <a class="btn btn-secondary btn-sm" href="/copy_jdk_configs/history/copy_jdk_configs_fixFiles.sh?identifier=88d3ed89f30d8b0eb4877d860fa8d951f224f156" title="View git log for this file">History</a>

                <a class="btn btn-secondary btn-sm" href="/copy_jdk_configs/raw/88d3ed89f30d8b0eb4877d860fa8d951f224f156/f/copy_jdk_configs_fixFiles.sh" title="View as raw">Raw</a>
            </div>

        <pre class="syntaxhighlightblock"><code class="">#!/bin/bash
config=$1
target=$2

debug(){
  if [ &#34;x$debug&#34; == &#34;xtrue&#34; ] ; then
    echo &#34;$@&#34;
  fi
}

debug &#34;cjc: bash debug is on&#34;

cmdvDebug() {
  if [ &#34;x$debug&#34; == &#34;xtrue&#34; ] ; then
    &#34;$@&#34; -v
  else
    &#34;$@&#34; 1&gt;/dev/null 2&gt;&amp;1
  fi
}

mvDebug() {
  cmdvDebug mv &#34;$@&#34;
}

rmDebug() {
  cmdvDebug rm &#34;$@&#34;
}

rmdirDebug() {
 cmdvDebug rmdir &#34;$@&#34;
}

#we should be pretty strict about removing once used (even &#34;used&#34; [with fail]) config file, as it may corrupt another installation
clean(){
  debug &#34;cleanup: removing $config&#34;
  rmDebug -rf $config
}

if [ &#34;x&#34; == &#34;x$config&#34; ] ; then
  debug &#34;no config file specified&#34;
  exit 1
fi

if [ ! -f  &#34;$config&#34; ] ; then
  debug &#34;$config file do not exists&#34;
  # expected case, when no migration happened
  exit 0
fi 

if [ &#34;x&#34; == &#34;x$target&#34; ] ; then
  debug &#34;no target dir specified&#34;
  clean
  exit 2
fi

if [ ! -d  &#34;$target&#34; ] ; then
  debug &#34;$target is not directory&#34;
  clean
  exit 22
fi 

source=`cat $config` 

if [ &#34;x&#34; == &#34;x$source&#34; ] ; then
  debug &#34;no information in $config&#34;
  clean
  exit 3
fi

if [ ! -d  &#34;$source&#34; ] ; then
  debug &#34;$source from $config is not directory&#34;
  clean
  exit 33
fi 


listLinks(){
  find $1 -type l -print0 | xargs -0 ls -ld | sed &#34;s; \+-&gt; \+;_-&gt;_;g&#34; | sed &#34;s;.* $1;$1;&#34;
}

printIfExists(){
  if [ -e $ffileCandidate ] ; then
    echo $1
  else
    # stdout can be captured, therefore stderr
    debug &#34;skipping not-existing link-target-dir $1&#34; 1&gt;&amp;2
  fi
}

createListOfLinksTargetsDirectories(){
  pushd $source &gt;/dev/null 2&gt;&amp;1 
    local links=`listLinks $1`
    for x in $links ; do 
      echo &#34;$x&#34; | grep &#34;jre-abrt&#34; &gt; /dev/null
      if [ $? -eq 0 ] ; then
        continue
      fi
      local ffileCandidate=$(echo $x | sed &#34;s/.*_-&gt;_//&#34;) ;
# ignoring relative paths as they may lead who know where later   
# there can be simlink relative to position, so push is not catching all
      if [ &#34;$ffileCandidate&#34; != &#34;${ffileCandidate#/}&#34; ] ; then
        if [ -d $ffileCandidate ] ; then
# should we accept the links to directories themselves?
          printIfExists $ffileCandidate
        else
          printIfExists `dirname $ffileCandidate`
        fi
      fi
    done | sort | uniq
  popd &gt;/dev/null 2&gt;&amp;1 
}

sourceLinks=`listLinks $source`
targetLinks=`listLinks $target`
sourceLinksDirsTarget=`createListOfLinksTargetsDirectories  $source`
targetLinksDirsTarget=`createListOfLinksTargetsDirectories  $target`

debug &#34;source: $source&#34;
debug &#34;target: $target&#34;

debug &#34;sourceLinks:
$sourceLinks&#34;
debug &#34;targetLinks:
$targetLinks&#34;

debug &#34;sourceLinksDirsTarget:
$sourceLinksDirsTarget&#34;
debug &#34;targetLinksDirsTarget:
$targetLinksDirsTarget&#34;

sourceSearchPath=&#34;$source $sourceLinksDirsTarget&#34;
targetSearchPath=&#34;$target $targetLinksDirsTarget&#34;

work(){
  if [ &#34;X$1&#34; == &#34;Xrpmnew&#34; -o &#34;X$1&#34; == &#34;Xrpmorig&#34; ] ; then
    debug &#34;Working with $1 (1)&#34;
  else
    debug &#34;unknown parameter: $1&#34;
    return 1
  fi

  local files=`find $targetSearchPath | grep &#34;\\.$1$&#34;`
  for file in $files ; do
    local sf1=`echo $file | sed &#34;s/\\.$1$//&#34;`
    local sf2=`echo $sf1 | sed &#34;s/$targetName/$srcName/&#34;`
    # was file modified in origianl installation?
    rpm -Vf $source | grep -q $sf2
    if [ $? -gt 0 ] ; then
     if [ &#34;X$1&#34; == &#34;Xrpmnew&#34; ] ; then
       debug &#34;$sf2 was NOT modified, removing possibly corrupted $sf1 and renaming from $file&#34;
       mvDebug -f $file $sf1
       if [ $? -eq 0 ] ; then
         echo &#34;restored $file to $sf1&#34;
       else
         debug &#34;FAILED to restore $file to $sf1&#34;
       fi
    fi
     if [ &#34;X$1&#34; == &#34;Xrpmorig&#34; ] ; then
       debug &#34;$sf2 was NOT modified, removing possibly corrupted $file&#34;
       rmDebug $file
    fi
    else
     debug &#34;$sf2 was modified, keeping $file, and removing the duplicated original&#34;
     # information is now backuped, in new directory anyway. Removing future rpmsave to allow rpm -e
     rmDebug -f $sf2
     # or its corresponding backup
     rmDebug -f $sf2.$1
    fi
done
}


srcName=`basename $source`
targetName=`basename $target`

work rpmnew
work rpmorig

debug &#34;Working with rpmorig (2)&#34;
# simply moving old rpmsaves to new dir
# fix for config (replace) leftovers
files=`find $sourceSearchPath | grep &#34;\\.rpmorig$&#34;`
  for file in $files ; do
    rpmsaveTarget=`echo $file | sed &#34;s/$srcName/$targetName/&#34;`
    debug &#34;relocating $file to $rpmsaveTarget&#34;
    if [ -e $rpmsaveTarget ] ; then
      rmDebug $file
    else
      mvDebug $file $rpmsaveTarget
    fi
  done

debug &#34;Working with rpmsave (1)&#34;
files=`find $sourceSearchPath | grep &#34;\\.rpmsave$&#34;`
  for file in $files ; do
    rpmsaveTarget=`echo $file | sed &#34;s/$srcName/$targetName/&#34;`
    debug &#34;relocating $file to $rpmsaveTarget&#34;
    if [ -e $rpmsaveTarget ] ; then
      rmDebug $file
    else
      mvDebug $file $rpmsaveTarget
    fi
  done


#warning: file /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.131-11.b12.el7.x86_64-debug/jre/lib/applet: remove failed: No such file or directory
#warning: file /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.131-11.b12.el7.x86_64-debug/jre/lib/amd64/client: remove failed: No such file or directory
#warning: file /usr/lib/jvm/java-1.7.0-openjdk-1.7.0.171-2.6.13.2.el7.x86_64/jre/lib/amd64/xawt: remove failed: No such file or directory
#those dirs might be mepty by installtion, filling to not be rmeoved later
#use exported CJC_BLACKDIRS_ADD to extend it in runtime/spec file
blackdirs=&#34;&#34;
internal_blackdirs=&#34;jre/lib/applet jre/lib/*/client jre/lib/locale/*/LC_MESSAGES jre/lib/*/xawt jre/javaws properties/version properties jre/lib/endorsed jre/lib/boot lib/missioncontrol/p2/org.eclipse.equinox.p2.engine/profileRegistry/JMC.profile/.data&#34;
for x in $internal_blackdirs $CJC_BLACKDIRS_ADD ; do 
  blackdirs=&#34;$blackdirs $source/$x&#34;
done

for blackdir in $blackdirs; do
  if [ -e $blackdir ] ; then
    debug &#34;nasty $blackdir  exists, filling&#34;
    touch $blackdir/C-J-C_placeholder
  else
    debug &#34;nasty $blackdir  DONT exists, ignoring&#34;
  fi
done

debug &#34;cleaning legacy leftowers&#34;
if [ &#34;x$debug&#34; == &#34;xtrue&#34; ] ; then
  find $sourceSearchPath -empty -type d -delete
else
  find $sourceSearchPath -empty -type d -delete 2&gt;/dev/null &gt;/dev/null
fi
rmdirDebug $sourceSearchPath

# and remove placeholders
for blackdir in $blackdirs; do
  if [ -e $blackdir ] ; then
    debug &#34;nasty $blackdir  exists, cleaning placeholder&#34;
    rmDebug $blackdir/C-J-C_placeholder
  else
    debug &#34;nasty $blackdir  DONT exists, ignoring again&#34;
  fi
done

clean

exit 0
</code></pre>
  </div>
 </div> <!-- end .card-->

</div>
</div>
</div>
    </div>

        <div class="footer pt-4 text-white">
        <div class="container">
            <div class="d-flex align-items-center">
                <div>
                    <div>Powered by <a href="https://pagure.io/pagure" class="notblue">Pagure</a> 5.13.3</div>
                    <div>
                        <a href="https://docs.pagure.org/pagure/usage/index.html" class="notblue">Documentation</a> &bull;
                        <a href="https://pagure.io/pagure/new_issue" class="notblue">File an Issue</a> &bull;
                        <a href="/about">About</a> &bull;
                        <a href="/ssh_info" class="notblue">SSH Hostkey/Fingerprint</a>
                    </div>
                </div>
                <div class="ml-auto text-right">
                    <div>&copy; Red Hat, Inc. and others.</div>
                </div>
            </div>
        </div>
    </div>


    <script type="text/javascript" nonce="wzrWagCkCunEewlwGprvZww2I" src="/static/vendor/jquery/jquery.min.js?version=5.13.3"></script>

    <script src="/static/vendor/bootstrap/bootstrap.bundle.min.js?version=5.13.3"></script>

    <script type="text/javascript" nonce="wzrWagCkCunEewlwGprvZww2I">
      $('[data-toggle="tooltip"]').tooltip({placement : 'bottom'});
      $(".cancel_btn").click(function() {
        history.back();
      });
    </script>

<script type="text/javascript"  nonce="wzrWagCkCunEewlwGprvZww2I" src="/static/vendor/lazyload/lazyload.min.js?version=5.13.3"></script>

<script type="text/javascript" nonce="wzrWagCkCunEewlwGprvZww2I">
window.addEventListener("load", function(event) {
    lazyload();
});
</script>

<script type="text/javascript" nonce="wzrWagCkCunEewlwGprvZww2I">
$("#giturl-toggle").on('click', function(event){
  event.stopPropagation();
  $("#giturl-more").toggle();
  $("#giturl-toggle").hide();
})

$(".fork_project_btn").click(function() {
  $('#fork_project').submit();
});

$(".select-on-focus").on("focus", function() {
  $(this).select();
});

</script>

<script type="text/javascript" nonce="wzrWagCkCunEewlwGprvZww2I" src="/static/vendor/highlight.js/highlight.pack.js?version=5.13.3"></script>
<script type="text/javascript" nonce="wzrWagCkCunEewlwGprvZww2I" src="/static/vendor/highlightjs-line-numbers/highlightjs-line-numbers.min.js?version=5.13.3"></script>
<script type="text/javascript" nonce="wzrWagCkCunEewlwGprvZww2I" src="/static/vendor/highlight.js/spec.js?version=5.13.3"></script>

<script type="text/javascript" nonce="wzrWagCkCunEewlwGprvZww2I">
  $(document).ready(function() {
  $('.fork_project_btn').click($("[name=fork_project]").submit);

  $('pre.syntaxhighlightblock code').each(function(i, block) {
    hljs.highlightBlock(block);
    hljs.lineNumbersBlock(block);
  });

  var cls = "highlighted-line";
  var lines = location.hash.substr(2).split('-').map(function (x) { return parseInt(x, 10) });
  if (! isNaN(lines[0]))
  {
    for (var i = lines[lines.length - 1]; i >= lines[0]; i--) {
      $('#_' + i).parent().parent().addClass(cls);
    }
    setTimeout(function(){
      $("#_" + lines[0]).get(0).scrollIntoView({behavior: "instant", block: "start", inline: "nearest"});
    }, 50);
  }
});
</script>

<script type="text/javascript" nonce="wzrWagCkCunEewlwGprvZww2I">

  function updateHighlight() {
    var cls = "highlighted-line";
    $('.' + cls).removeClass(cls)
    if (location.hash !== '') {
      var lines = location.hash.substr(2).split('-').map(function (x) { return parseInt(x, 10) });
      for (var i = lines[lines.length - 1]; i >= lines[0]; i--) {
        $('[data-line-number=' + i + ']').closest('tr').addClass(cls);
      }
      return lines;
    }
    return [];
  }
  $(window).on('hashchange', updateHighlight);
  var selected = [];
  $("[data-line-number]").click(function (ev) {
    var line = $(this).attr('data-line-number');
    if (ev.shiftKey) {
      selected = selected.slice(-1).concat(line);
    } else {
      selected = [line];
    }

    var hash = '_' + selected[0];
    if (selected.length === 2) {
      hash = '_' + Math.min(selected[0], selected[1]) + '-' + Math.max(selected[0], selected[1]);
    }
    window.location.hash = hash;
    return false;
  });

</script>


</body>
</html>